#include <stdio.h>
#include <math.h>

/* routine to compute nth_root of input float vector */
void compress(float *fvec, int length, float nth_root)
{
	int i;
	float eps = .000001;
	float the_power;

	the_power = 1. / nth_root;

	for(i=0; i<length; i++)
	{
		if(fvec[i] < eps)fvec[i] = eps;
		fvec[i] = pow(fvec[i], (double)the_power);
	}
}

/* Take log and divide by 3 (like taking cube root, but end up
	in log domain for further processing first) */
void lcompress(float *fvec, int length)
{
	int i;
	float eps = .000001;

	for(i=0; i<length; i++)
	{
		if(fvec[i] < eps)fvec[i] = eps;
		fvec[i] = log(fvec[i])/ 3.0;
	}
}

/* saturated exponentiation of float vector */
void expand(float *fvec, int length)
{
	int i;
	float big = 20.;

	for(i=0; i<length; i++)
	{
		if(fvec[i] > big)fvec[i] = big;
		if(fvec[i] < (-big))fvec[i] = (-big);
		fvec[i] = exp(fvec[i]);
	}
}

/* float vector move; note that destination is first in args */
void vecmove(float *destvec, float *sourcevec, int length)
{
	int i;

	for(i=0; i<length; i++)
	{
		destvec[i] = sourcevec[i];
	}
}

/* divide the mean from the vector */
void mean_div(float *fvec, int length)
{
	int i;
	float mean = 0.;

	for(i=0; i<length; i++)
	{
		mean += fvec[i];
	}
	mean /= (float)length;

	for(i=0; i<length; i++)
	{
		fvec[i] /= mean;
	}
}
/* subtract off the minimum value and scale everything so max is 1.0
*/
void fix_range(float *fvec, int length)
{
	int i;
	float max = 0.;
	float min = 1000000.;

	for(i=0; i<length; i++)
	{
		if(fvec[i] > max) max = fvec[i];
		if(fvec[i] < min) min = fvec[i];
	}

	max -= min;

	for(i=0; i<length; i++)
	{
		fvec[i] -= min;
		fvec[i] /= max;
	}
}

/* remove the mean from the vector */
void mean_rem(float *fvec, int length)
{
	int i;
	float mean = 0.;

	for(i=0; i<length; i++)
	{
		mean += fvec[i];
	}
	mean /= (float)length;

	for(i=0; i<length; i++)
	{
		fvec[i] -= mean;
	}
}


/* Get the local slope estimates for a vector */
void getdelta(float *fvec, int length)
{
	int i,j;
	float slope = 0.;
	float * ftmp = (float*) calloc(10000,sizeof(float));
	// float ftmp[10000];

	if(length > 10000)
	{
		fprintf(stderr,"length of %d too big for getdelta\n");
		exit(-1);
	}

	for(i=4; i<(length-4); i++)
	{
		for(j=-4; j<=4; j++)
		{
			slope += (j * fvec[i+j]);
		}
		slope /= 60.;
		ftmp[i] = slope;
	}
	fvec[0] = fvec[1] = ftmp[2];
	fvec[length-2] = fvec[length-1] = ftmp[length-3];

	for(i=2; i<(length-2); i++)
	{
		fvec[i] = ftmp[i];
	}
	free(ftmp);
}

/* normalize the average absolute deviation from the mean */
void norm_dev(float *fvec, int length)
{
	int i;
	float mean = 0.;
	float dev = 0.;

	for(i=0; i<length; i++)
	{
		mean += fvec[i];
	}
	mean /= (float)length;
	for(i=0; i<length; i++)
	{
		dev += fabs(fvec[i] - mean);
	}
	dev /= (float)length;
	for(i=0; i<length; i++)
	{
		fvec[i] /= dev;
	}
}

/* half-rectify (set negative values to zero) a float vector */
void half_rect(float *fvec, int length)
{
	int i;

	for(i=0; i<length; i++)
	{
		if(fvec[i] < 0.)
		{
			fvec[i] = 0.;
		}
	}
}

/* This routine reads named file  of ascii floats*/
int readfvec(float *fvec, char *fname )
{
	int j, nread;

	FILE *fp;

	fp = fopen(fname, "r");
	if(fp == (FILE *) NULL)
	{
		fprintf(stderr,"Cannot open file %s \n",
			fname);
		exit(-1);
	}

	j = 0;
	while((nread = fscanf(fp,"%g", &fvec[j])) == 1)
	{
		j++;
	}

	return(j);
}
